@@ -31,13 +31,16 @@ from pyomo.environ import *
 
 from six import iteritems
 
+#DEBUG
+from nose.tools import set_trace
+
 try:
     import yaml
     yaml_available=True
 except ImportError:
     yaml_available=False
 
-solvers = pyomo.opt.check_available_solvers('cplex', 'glpk')
+solvers = pyomo.opt.check_available_solvers('cplex', 'glpk','gurobi')
 
 
 if False:
@@ -58,7 +61,8 @@ if False:
                         original = getattr(base, key, None)
                         if original is not None:
                             copy = copyfunc(original)
-                            copy.__doc__ = attrs[key].__doc__ + " (%s)" % copy.__name__
+                            copy.__doc__ = attrs[key].__doc__ + \
+                                           " (%s)" % copy.__name__
                             attrs[key] = copy
                             break
             for base in bases:
@@ -90,8 +94,16 @@ class CommonTests:
             pp = kwds['preprocess']
             if pp == 'bigm':
                 args.append('--transform=gdp.bigm')
+                # ESJ: HACK for now: also apply the reclassify
+                # transformation in this case
+                args.append('--transform=gdp.reclassify')
             elif pp == 'chull':
                 args.append('--transform=gdp.chull')
+                # ESJ: HACK for now: also apply the reclassify
+                # transformation in this case
+                args.append('--transform=gdp.reclassify')
+            elif pp == 'cuttingplane':
+                args.append('--transform=gdp.cuttingplane')
         args.append('-c')
         args.append('--symbolic-solver-labels')
         os.chdir(currdir)
@@ -135,6 +147,9 @@ class CommonTests:
         # Run the small jobshop example using the BigM transformation
         self.pyomo( join(exdir,'jobshop.py'), join(exdir,'jobshop-small.dat'),
                     preprocess='bigm' )
+        # ESJ: TODO: Right now the indicator variables have names they won't
+        # have when they don't have to be reclassified. So I think this LP file
+        # will need to change again.
         self.check( 'jobshop_small', 'bigm' )
 
     def test_bigm_jobshop_large(self):
@@ -142,8 +157,18 @@ class CommonTests:
         # Run the large jobshop example using the BigM transformation
         self.pyomo( join(exdir,'jobshop.py'), join(exdir,'jobshop.dat'),
                     preprocess='bigm')
+        # ESJ: TODO: this LP file also will need to change with the
+        # indicator variable change.
         self.check( 'jobshop_large', 'bigm' )
 
+    # def test_bigm_constrained_layout(self):
+    #     self.problem='test_bigm_constrained_layout'
+    #     # Run the constrained layout example with the bigm transformation
+    #     self.pyomo( join(exdir,'ConstrainedLayout.py'), 
+    #                 join(exdir,'ConstrainedLayout_BigM.dat'), 
+    #                 preprocess='bigm', solver='cplex')
+    #     self.check( 'constrained_layout', 'bigm')
+
     def test_chull_jobshop_small(self):
         self.problem='test_chull_jobshop_small'
         # Run the small jobshop example using the CHull transformation
@@ -158,6 +183,22 @@ class CommonTests:
                     preprocess='chull')
         self.check( 'jobshop_large', 'chull' )
 
+    @unittest.skip("cutting plane LP file tests are too fragile")
+    @unittest.skipIf('gurobi' not in solvers, 'Gurobi solver not available')
+    def test_cuttingplane_jobshop_small(self):
+        self.problem='test_cuttingplane_jobshop_small'
+        self.pyomo( join(exdir,'jobshop.py'), join(exdir,'jobshop-small.dat'),
+                    preprocess='cuttingplane')
+        self.check( 'jobshop_small', 'cuttingplane' )
+
+    @unittest.skip("cutting plane LP file tests are too fragile")
+    @unittest.skipIf('gurobi' not in solvers, 'Gurobi solver not available')
+    def test_cuttingplane_jobshop_large(self):
+        self.problem='test_cuttingplane_jobshop_large'
+        self.pyomo( join(exdir,'jobshop.py'), join(exdir,'jobshop.dat'),
+                    preprocess='cuttingplane')
+        self.check( 'jobshop_large', 'cuttingplane' )
+
 
 class Reformulate(unittest.TestCase, CommonTests):
 
@@ -193,7 +234,11 @@ class Solver(unittest.TestCase):
         for i in range(len(refObj)):
             self.assertEqual(len(refObj[i]), len(ansObj[i]))
             for key,val in iteritems(refObj[i]):
-                self.assertEqual(val, ansObj[i].get(key,None))
+                self.assertAlmostEqual(
+                    val.get('Value', None),
+                    ansObj[i].get(key,{}).get('Value', None),
+                    6
+                )
 
 
 @unittest.skipIf(not yaml_available, "YAML is not available")
@@ -206,7 +251,8 @@ class Solve_GLPK(Solver, CommonTests):
 
 
 @unittest.skipIf(not yaml_available, "YAML is not available")
-@unittest.skipIf(not 'cplex' in solvers, "The 'cplex' executable is not available")
+@unittest.skipIf(not 'cplex' in solvers, 
+                 "The 'cplex' executable is not available")
 class Solve_CPLEX(Solver, CommonTests):
 
     def pyomo(self,  *args, **kwds):
