@@ -8,26 +8,25 @@
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
 
-from six import StringIO, iteritems, itervalues
+from six import StringIO, iteritems, string_types
 from tempfile import mkdtemp
 import os, sys, math, logging, shutil, time
 
-from pyomo.core.base import (Constraint, Suffix, Var, value,
-                             Expression, Objective)
+from pyomo.core.base import Constraint, Var, value, Objective
 from pyomo.opt import ProblemFormat, SolverFactory
 
-import pyomo.util.plugin
-from pyomo.opt.base import IOptSolver
-import pyutilib.services
+import pyomo.common
 
 from pyomo.opt.base.solvers import _extract_version
 import pyutilib.subprocess
-from pyutilib.misc import Options
+from pyutilib.misc import Options, quote_split
 
-from pyomo.core.kernel.component_block import IBlockStorage
+from pyomo.core.kernel.block import IBlock
+from pyomo.core.kernel.objective import IObjective
+from pyomo.core.kernel.variable import IVariable
 
 import pyomo.core.base.suffix
-import pyomo.core.kernel.component_suffix
+import pyomo.core.kernel.suffix
 
 from pyomo.opt.results import (SolverResults, SolverStatus, Solution,
     SolutionStatus, TerminationCondition, ProblemSense)
@@ -35,21 +34,89 @@ from pyomo.opt.results import (SolverResults, SolverStatus, Solution,
 
 logger = logging.getLogger('pyomo.solvers')
 
-pyutilib.services.register_executable(name="gams")
+pyomo.common.register_executable(name="gams")
 
-class GAMSSolver(pyomo.util.plugin.Plugin):
+class _GAMSSolver(object):
+    """Aggregate of common methods for GAMS interfaces"""
+
+    def __init__(self, **kwds):
+        self._version = None
+        self._default_variable_value = None
+        self._metasolver = False
+
+        self._capabilities = Options()
+        self._capabilities.linear = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
+        self._capabilities.integer = True
+        self._capabilities.sos1 = False
+        self._capabilities.sos2 = False
+
+        self.options = Options()
+
+    def version(self):
+        """Returns a 4-tuple describing the solver executable version."""
+        if self._version is None:
+            self._version = self._get_version()
+        return self._version
+
+    def warm_start_capable(self):
+        """True is the solver can accept a warm-start solution."""
+        return True
+
+    def default_variable_value(self):
+        return self._default_variable_value
+
+    def set_options(self, istr):
+        if isinstance(istr, string_types):
+            istr = self._options_string_to_dict(istr)
+        for key in istr:
+            if not istr[key] is None:
+                setattr(self.options, key, istr[key])
+
+    @staticmethod
+    def _options_string_to_dict(istr):
+        ans = {}
+        istr = istr.strip()
+        if not istr:
+            return ans
+        if istr[0] == "'" or istr[0] == '"':
+            istr = eval(istr)
+        tokens = quote_split('[ ]+',istr)
+        for token in tokens:
+            index = token.find('=')
+            if index is -1:
+                raise ValueError(
+                    "Solver options must have the form option=value: '%s'" % istr)
+            try:
+                val = eval(token[(index+1):])
+            except:
+                val = token[(index+1):]
+            ans[token[:index]] = val
+        return ans
+
+    #
+    # Support "with" statements.
+    #
+    def __enter__(self):
+        return self
+
+    def __exit__(self, t, v, traceback):
+        pass
+
+
+
+@SolverFactory.register('gams', doc='The GAMS modeling language')
+class GAMSSolver(_GAMSSolver):
     """
-    A generic interface to GAMS solvers
+    A generic interface to GAMS solvers.
 
     Pass solver_io keyword arg to SolverFactory to choose solver mode:
         solver_io='direct' or 'python' to use GAMS Python API
-            Requires installation, visit https://www.gams.com for help.
+            Requires installation, visit Python API page on gams.com for help.
         solver_io='shell' or 'gms' to use command line to call gams
             Requires the gams executable be on your system PATH.
     """
-    pyomo.util.plugin.implements(IOptSolver)
-    pyomo.util.plugin.alias('gams', doc='The GAMS modeling language')
-
     def __new__(cls, *args, **kwds):
         try:
             mode = kwds['solver_io']
@@ -68,29 +135,17 @@ class GAMSSolver(pyomo.util.plugin.Plugin):
             return
 
 
-class GAMSDirect(pyomo.util.plugin.Plugin):
-    """A generic interface to GAMS solvers"""
-    pyomo.util.plugin.implements(IOptSolver)
-    pyomo.util.plugin.alias('_gams_direct', doc='The GAMS modeling language')
+@SolverFactory.register('_gams_direct',
+        doc='Direct python interface to the GAMS modeling language')
+class GAMSDirect(_GAMSSolver):
+    """
+    A generic python interface to GAMS solvers.
 
-    def __init__(self, **kwds):
-        self._version = None
-        self._default_variable_value = None
-
-        self._capabilities = Options()
-        self._capabilities.linear = True
-        self._capabilities.quadratic_objective = True
-        self._capabilities.quadratic_constraint = True
-        self._capabilities.integer = True
-        self._capabilities.sos1 = False
-        self._capabilities.sos2 = False
-
-        self.options = Options() # ignored
-
-        pyomo.util.plugin.Plugin.__init__(self, **kwds)
+    Visit Python API page on gams.com for installation help.
+    """
 
     def available(self, exception_flag=True):
-        """True if the solver is available"""
+        """True if the solver is available."""
         try:
             from gams import GamsWorkspace, DebugLevel
             return True
@@ -103,82 +158,42 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
                                   "GAMS message: %s" % e)
 
     def _get_version(self):
-        """
-        Returns a tuple describing the solver executable version.
-        """
+        """Returns a tuple describing the solver executable version."""
         if not self.available(exception_flag=False):
             return _extract_version('')
         from gams import GamsWorkspace
         ws = GamsWorkspace()
-        version = tuple(int(i) for i in ws._version.split('.'))
+        version = tuple(int(i) for i in ws._version.split('.')[:4])
         while(len(version) < 4):
             version += (0,)
-        version = version[:4]
         return version
 
-    def version(self):
-        """
-        Returns a 4-tuple describing the solver executable version.
-        """
-        if self._version is None:
-            self._version = self._get_version()
-        return self._version
-
-    def warm_start_capable(self):
-        return True
-
-    def default_variable_value(self):
-        return self._default_variable_value
-
     def solve(self, *args, **kwds):
         """
-        Uses GAMS Python API. Visit https://www.gams.com for installation help.
+        Solve a model via the GAMS Python API.
 
-        tee=False:
+        Keyword Arguments
+        -----------------
+        tee=False: bool
             Output GAMS log to stdout.
-        logfile=None:
-            Optionally a logfile can be written.
-        load_solutions=True:
-            Optionally skip loading solution into model, in which case
-            the results object will contain the solution data.
-        keepfiles=False:
+        logfile=None: str
+            Filename to output GAMS log to a file.
+        load_solutions=True: bool
+            Load solution into model. If False, the results
+            object will contain the solution data.
+        keepfiles=False: bool
             Keep temporary files. Equivalent of DebugLevel.KeepFiles.
             Summary of temp files can be found in _gams_py_gjo0.pf
-        tmpdir=None:
+        tmpdir=None: str
             Specify directory path for storing temporary files.
             A directory will be created if one of this name doesn't exist.
-            None (default) uses the system default temporary path.
-        report_timing=False:
+            By default uses the system default temporary path.
+        report_timing=False: bool
             Print timing reports for presolve, solver, postsolve, etc.
-        io_options:
-            Updated with additional keywords passed to solve()
-            warmstart=False:
-                Warmstart by initializing model's variables to their values.
-            symbolic_solver_labels=False:
-                Use full Pyomo component names rather than
-                shortened symbols (slower, but useful for debugging).
-            labeler=None:
-                Custom labeler option. Incompatible with symbolic_solver_labels.
-            solver=None:
-                If None, GAMS will use default solver for model type.
-            mtype=None:
-                Model type. If None, will chose from lp, nlp, mip, and minlp.
-            add_options=None:
-                List of additional lines to write directly
-                into model file before the solve statement.
-                For model attributes, <model name> is GAMS_MODEL.
-            skip_trivial_constraints=False:
-                Skip writing constraints whose body section is fixed
-            file_determinism=1:
-                How much effort do we want to put into ensuring the
-                GAMS file is written deterministically for a Pyomo model:
-                   0 : None
-                   1 : sort keys of indexed components (default)
-                   2 : sort keys AND sort names (over declaration order)
-            put_results=None:
-                Filename for optionally writing solution values and
-                marginals to (put_results).dat, and solver statuses
-                to (put_results + 'stat').dat.
+        io_options: dict
+            Options that get passed to the writer.
+            See writer in pyomo.repn.plugins.gams_writer for details.
+            Updated with any other keywords passed to solve method.
         """
 
         # Make sure available() doesn't crash
@@ -192,18 +207,22 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
                              'to solve method of GAMSSolver.')
         model = args[0]
 
-        load_solutions = kwds.pop("load_solutions", True)
-        tee            = kwds.pop("tee", False)
-        logfile        = kwds.pop("logfile", None)
-        keepfiles      = kwds.pop("keepfiles", False)
-        tmpdir         = kwds.pop("tmpdir", None)
-        report_timing  = kwds.pop("report_timing", False)
-        io_options     = kwds.pop("io_options", {})
+        # self.options are default for each run, overwritten by kwds
+        options = dict()
+        options.update(self.options)
+        options.update(kwds)
 
-        if len(kwds):
-            # Pass remaining keywords to writer, which will handle
-            # any unrecognized arguments
-            io_options.update(kwds)
+        load_solutions = options.pop("load_solutions", True)
+        tee            = options.pop("tee", False)
+        logfile        = options.pop("logfile", None)
+        keepfiles      = options.pop("keepfiles", False)
+        tmpdir         = options.pop("tmpdir", None)
+        report_timing  = options.pop("report_timing", False)
+        io_options     = options.pop("io_options", {})
+
+        # Pass remaining keywords to writer, which will handle
+        # any unrecognized arguments
+        io_options.update(options)
 
         initial_time = time.time()
 
@@ -215,7 +234,7 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
         # model file will be written. The writer also passes this StringIO
         # back, but output_file is defined in advance for clarity.
         output_file = StringIO()
-        if isinstance(model, IBlockStorage):
+        if isinstance(model, IBlock):
             # Kernel blocks have slightly different write method
             smap_id = model.write(filename=output_file,
                                   format=ProblemFormat.gams,
@@ -293,13 +312,12 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
         ####################################################################
 
         # import suffixes must be on the top-level model
-        if isinstance(model, IBlockStorage):
-            model_suffixes = list(name for (name,comp) \
-                                  in pyomo.core.kernel.component_suffix.\
+        if isinstance(model, IBlock):
+            model_suffixes = list(comp.storage_key for comp \
+                                  in pyomo.core.kernel.suffix.\
                                   import_suffix_generator(model,
                                                           active=True,
-                                                          descend_into=False,
-                                                          return_key=True))
+                                                          descend_into=False))
         else:
             model_suffixes = list(name for (name,comp) \
                                   in pyomo.core.base.suffix.\
@@ -436,11 +454,11 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
 
         for sym, ref in iteritems(symbolMap.bySymbol):
             obj = ref()
-            if isinstance(model, IBlockStorage):
+            if isinstance(model, IBlock):
                 # Kernel variables have no 'parent_component'
-                if obj.ctype is Objective:
+                if obj.ctype is IObjective:
                     soln.objective[sym] = {'Value': objctvval}
-                if obj.ctype is not Var:
+                if obj.ctype is not IVariable:
                     continue
             else:
                 if obj.parent_component().type() is Objective:
@@ -506,7 +524,7 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
 
         results._smap_id = smap_id
         results._smap = None
-        if isinstance(model, IBlockStorage):
+        if isinstance(model, IBlock):
             if len(results.solution) == 1:
                 results.solution(0).symbol_map = \
                     getattr(model, "._symbol_maps")[results._smap_id]
@@ -544,30 +562,14 @@ class GAMSDirect(pyomo.util.plugin.Plugin):
         return results
 
 
-class GAMSShell(pyomo.util.plugin.Plugin):
-    """A generic interface to GAMS solvers"""
-    pyomo.util.plugin.implements(IOptSolver)
-    pyomo.util.plugin.alias('_gams_shell', doc='The GAMS modeling language')
-
-    def __init__(self, **kwds):
-        self._version = None
-        self._default_variable_value = None
-
-        self._capabilities = Options()
-        self._capabilities.linear = True
-        self._capabilities.quadratic_objective = True
-        self._capabilities.quadratic_constraint = True
-        self._capabilities.integer = True
-        self._capabilities.sos1 = False
-        self._capabilities.sos2 = False
-
-        self.options = Options() # ignored
-
-        pyomo.util.plugin.Plugin.__init__(self, **kwds)
+@SolverFactory.register('_gams_shell',
+        doc='Shell interface to the GAMS modeling language')
+class GAMSShell(_GAMSSolver):
+    """A generic shell interface to GAMS solvers."""
 
     def available(self, exception_flag=True):
-        """True if the solver is available"""
-        exe = pyutilib.services.registered_executable("gams")
+        """True if the solver is available."""
+        exe = pyomo.common.registered_executable("gams")
         if exception_flag is False:
             return exe is not None
         else:
@@ -579,7 +581,7 @@ class GAMSShell(pyomo.util.plugin.Plugin):
                     "solver functionality is not available.")
 
     def _default_executable(self):
-        executable = pyutilib.services.registered_executable("gams")
+        executable = pyomo.common.registered_executable("gams")
         if executable is None:
             logger.warning("Could not locate the 'gams' executable, "
                            "which is required for solver gams")
@@ -588,83 +590,50 @@ class GAMSShell(pyomo.util.plugin.Plugin):
         return executable.get_path()
 
     def executable(self):
-        """
-        Returns the executable used by this solver.
-        """
+        """Returns the executable used by this solver."""
         return self._default_executable()
 
     def _get_version(self):
-        """
-        Returns a tuple describing the solver executable version.
-        """
+        """Returns a tuple describing the solver executable version."""
         solver_exec = self.executable()
 
         if solver_exec is None:
             return _extract_version('')
         else:
-            results = pyutilib.subprocess.run([solver_exec])
-            return _extract_version(results[1])
-
-    def version(self):
-        """
-        Returns a 4-tuple describing the solver executable version.
-        """
-        if self._version is None:
-            self._version = self._get_version()
-        return self._version
-
-    def warm_start_capable(self):
-        return True
-
-    def default_variable_value(self):
-        return self._default_variable_value
+            # specify logging to stdout for windows compatibility
+            # technically this command makes gams complain because we're not
+            # providing a filename, but it will include the version name anyway
+            cmd = [solver_exec, "", "lo=3"]
+            _, txt = pyutilib.subprocess.run(cmd, tee=False)
+            return _extract_version(txt)
 
     def solve(self, *args, **kwds):
         """
-        Uses command line to call GAMS.
+        Solve a model via the GAMS executable.
 
-        tee=False:
+        Keyword Arguments
+        -----------------
+        tee=False: bool
             Output GAMS log to stdout.
-        logfile=None:
-            Optionally a logfile can be written.
-        load_solutions=True:
-            Optionally skip loading solution into model, in which case
-            the results object will contain the solution data.
-        keepfiles=False:
+        logfile=None: str
+            Filename to output GAMS log to a file.
+        load_solutions=True: bool
+            Load solution into model. If False, the results
+            object will contain the solution data.
+        keepfiles=False: bool
             Keep temporary files.
-        tmpdir=None:
+        tmpdir=None: str
             Specify directory path for storing temporary files.
             A directory will be created if one of this name doesn't exist.
-            None (default) uses the system default temporary path.
-        report_timing=False:
+            By default uses the system default temporary path.
+        report_timing=False: bool
             Print timing reports for presolve, solver, postsolve, etc.
-        io_options:
-            Updated with additional keywords passed to solve()
-            warmstart=False:
-                Warmstart by initializing model's variables to their values.
-            symbolic_solver_labels=False:
-                Use full Pyomo component names rather than
-                shortened symbols (slower, but useful for debugging).
-            labeler=None:
-                Custom labeler. Incompatible with symbolic_solver_labels.
-            solver=None:
-                If None, GAMS will use default solver for model type.
-            mtype=None:
-                Model type. If None, will chose from lp, nlp, mip, and minlp.
-            add_options=None:
-                List of additional lines to write directly
-                into model file before the solve statement.
-                For model attributes, <model name> is GAMS_MODEL.
-            skip_trivial_constraints=False:
-                Skip writing constraints whose body section is fixed
-            file_determinism=1:
-                How much effort do we want to put into ensuring the
-                GAMS file is written deterministically for a Pyomo model:
-                   0 : None
-                   1 : sort keys of indexed components (default)
-                   2 : sort keys AND sort names (over declaration order)
-            put_results='results':
-                Not available for modification on GAMSShell solver.
+        io_options: dict
+            Options that get passed to the writer.
+            See writer in pyomo.repn.plugins.gams_writer for details.
+            Updated with any other keywords passed to solve method.
+            Note: put_results is not available for modification on
+            GAMSShell solver.
         """
 
         # Make sure available() doesn't crash
@@ -675,19 +644,23 @@ class GAMSShell(pyomo.util.plugin.Plugin):
                              'to solve method of GAMSSolver.')
         model = args[0]
 
-        load_solutions = kwds.pop("load_solutions", True)
-        tee            = kwds.pop("tee", False)
-        logfile        = kwds.pop("logfile", None)
-        keepfiles      = kwds.pop("keepfiles", False)
-        tmpdir         = kwds.pop("tmpdir", None)
-        report_timing  = kwds.pop("report_timing", False)
-        io_options     = kwds.pop("io_options", {})
+        # self.options are default for each run, overwritten by kwds
+        options = dict()
+        options.update(self.options)
+        options.update(kwds)
 
-        if len(kwds):
-            # Pass remaining keywords to writer, which will handle
-            # any unrecognized arguments
-            io_options.update(kwds)
+        load_solutions = options.pop("load_solutions", True)
+        tee            = options.pop("tee", False)
+        logfile        = options.pop("logfile", None)
+        keepfiles      = options.pop("keepfiles", False)
+        tmpdir         = options.pop("tmpdir", None)
+        report_timing  = options.pop("report_timing", False)
+        io_options     = options.pop("io_options", {})
 
+        io_options.update(options)
+
+        # Pass remaining keywords to writer, which will handle
+        # any unrecognized arguments
         initial_time = time.time()
 
         ####################################################################
@@ -711,14 +684,17 @@ class GAMSShell(pyomo.util.plugin.Plugin):
             os.makedirs(tmpdir)
             newdir = True
 
-        output_filename = os.path.join(tmpdir, 'model.gms')
-        lst_filename = os.path.join(tmpdir, 'output.lst')
-        statresults_filename = os.path.join(tmpdir, 'resultsstat.dat')
+        output = "model.gms"
+        output_filename = os.path.join(tmpdir, output)
+        lst = "output.lst"
+        lst_filename = os.path.join(tmpdir, lst)
 
-        io_options['put_results'] = os.path.join(tmpdir, 'results')
-        results_filename = os.path.join(tmpdir, 'results.dat')
+        put_results = "results"
+        io_options["put_results"] = put_results
+        results_filename = os.path.join(tmpdir, put_results + ".dat")
+        statresults_filename = os.path.join(tmpdir, put_results + "stat.dat")
 
-        if isinstance(model, IBlockStorage):
+        if isinstance(model, IBlock):
             # Kernel blocks have slightly different write method
             smap_id = model.write(filename=output_filename,
                                   format=ProblemFormat.gams,
@@ -741,7 +717,7 @@ class GAMSShell(pyomo.util.plugin.Plugin):
         ####################################################################
 
         exe = self.executable()
-        command = [exe, output_filename, 'o=' + lst_filename]
+        command = [exe, output, "o=" + lst, "curdir=" + tmpdir]
         if tee and not logfile:
             # default behaviour of gams is to print to console, for
             # compatability with windows and *nix we want to explicitly log to
@@ -797,13 +773,12 @@ class GAMSShell(pyomo.util.plugin.Plugin):
         ####################################################################
 
         # import suffixes must be on the top-level model
-        if isinstance(model, IBlockStorage):
-            model_suffixes = list(name for (name,comp) \
-                                  in pyomo.core.kernel.component_suffix.\
+        if isinstance(model, IBlock):
+            model_suffixes = list(comp.storage_key for comp \
+                                  in pyomo.core.kernel.suffix.\
                                   import_suffix_generator(model,
                                                           active=True,
-                                                          descend_into=False,
-                                                          return_key=True))
+                                                          descend_into=False))
         else:
             model_suffixes = list(name for (name,comp) \
                                   in pyomo.core.base.suffix.\
@@ -952,11 +927,11 @@ class GAMSShell(pyomo.util.plugin.Plugin):
         has_rc_info = True
         for sym, ref in iteritems(symbolMap.bySymbol):
             obj = ref()
-            if isinstance(model, IBlockStorage):
+            if isinstance(model, IBlock):
                 # Kernel variables have no 'parent_component'
-                if obj.ctype is Objective:
+                if obj.ctype is IObjective:
                     soln.objective[sym] = {'Value': objctvval}
-                if obj.ctype is not Var:
+                if obj.ctype is not IVariable:
                     continue
             else:
                 if obj.parent_component().type() is Objective:
@@ -1025,7 +1000,7 @@ class GAMSShell(pyomo.util.plugin.Plugin):
 
         results._smap_id = smap_id
         results._smap = None
-        if isinstance(model, IBlockStorage):
+        if isinstance(model, IBlock):
             if len(results.solution) == 1:
                 results.solution(0).symbol_map = \
                     getattr(model, "._symbol_maps")[results._smap_id]
@@ -1141,7 +1116,7 @@ def check_expr(expr, name, solver_io):
     # Used to handle log and log10 violations, for example
     try:
         value(expr)
-    except ValueError:
+    except (ValueError, ZeroDivisionError):
         logger.warning("While evaluating model.%s's expression, GAMS solver "
                        "encountered an error.\nGAMS requires that all "
                        "equations and expressions evaluate at initial values.\n"
